import json
import locale

from flask import request, Response, jsonify

from heman.api import AuthorizedResource
from heman.auth import check_cups_allowed, check_contract_allowed
from heman.config import peek


TYPES_MAP = {
    'integer': 'integer',
    'char': 'string',
    'selection': 'string',
    'boolean': 'boolean'
}


def get_first_lang():
    """Get the first lang of Accept-Language Header.
    """
    request_lang = request.headers.get('Accept-Language').split(',')
    if request_lang:
        lang = locale.normalize(request_lang[0]).split('.')[0]
    else:
        lang = False
    return lang


def jsonform(definition):
    res = {'schema': {}, 'form': []}
    schema = res['schema']
    form = res['form']
    for name, fdef in definition.items():
        if fdef['type'] not in TYPES_MAP:
            continue
        schema[name] = {
            'type': TYPES_MAP[fdef['type']],
            'title': fdef['string'],
            'description': fdef.get('help'),
            'required': fdef.get('required', False)
        }
        selection = fdef.get('selection')
        if selection:
            enum = []
            title_map = {}
            for key, value in selection:
                enum.append(key)
                title_map[key] = value
            schema[name]['enum'] = enum
            form.append({
                'key': name,
                'titleMap': title_map
            })
        else:
            form.append({'key': name})
    form.append({'type': 'submit', 'title': 'Enviar'})
    return res


class AuthorizedByCupsResource(AuthorizedResource):
    method_decorators = (
        AuthorizedResource.method_decorators + [check_cups_allowed]
    )


class AuthorizedByContractResource(AuthorizedResource):
    method_decorators = (
        AuthorizedResource.method_decorators + [check_contract_allowed]
    )


class FormResource(AuthorizedResource):
    pass


class EmpoweringBuildingDefForm(FormResource):
    def get(self):
        lang = get_first_lang()
        model = peek.model('empowering.cups.building')
        def_fields = model.fields_get([], context={'lang': lang})
        def_fields.pop('meteringPointId', None)
        res = jsonform(def_fields)
        return Response(json.dumps(res), mimetype='application/json')


class EmpoweringBuildingForm(FormResource, AuthorizedByCupsResource):
    def get(self, cups):
        model = peek.model('empowering.cups.building')
        cups_ids = model.search([('cups_id.name', '=', cups)])
        if not cups_ids:
            res = {}
        else:
            res = model.read(cups_ids[0])
        return Response(json.dumps(res), mimetype='application/json')

    def post(self, cups):
        model = peek.model('empowering.cups.building')
        building_ids = model.search([('cups_id.name', '=', cups)])
        data = request.json
        if not building_ids:
            cups = peek.model('giscedata.cups.ps').search([
                ('name', '=', cups)
            ])
            if not cups:
                return jsonify({'status': 404, 'message': 'Not Found'}), 404
            data['cups_id'] = cups[0]
            model.create(data)
        else:
            model.write(building_ids, data)
        return jsonify({'status': 200, 'message': 'OK'}), 200


class EmpoweringProfileDefForm(FormResource):
    def get(self):
        lang = get_first_lang()
        model = peek.model('empowering.modcontractual.profile')
        def_fields = model.fields_get([], context={'lang': lang})
        def_fields.pop('femalePersonsNumber', None)
        def_fields.pop('malePersonsNumber', None)
        def_fields.pop('eduLevel_sec', None)
        def_fields.pop('eduLevel_noStudies', None)
        def_fields.pop('eduLevel_prim', None)
        def_fields.pop('eduLevel_uni', None)
        res = jsonform(def_fields)
        return Response(json.dumps(res), mimetype='application/json')


class EmpoweringProfileForm(FormResource, AuthorizedByContractResource):
    def get(self, contract):
        model = peek.model('empowering.modcontractual.profile')
        profile_ids = model.search([
            ('modcontractual_id.polissa_id.name', '=', contract)
        ])
        if not profile_ids:
            res = {}
        else:
            res = model.read(profile_ids[0])
        return Response(json.dumps(res), mimetype='application/json')

    def post(self, contract):
        model = peek.model('empowering.modcontractual.profile')
        profile_ids = model.search([
            ('modcontractual_id.polissa_id.name', '=', contract)
        ])
        data = request.json
        contract_obj = peek.model('giscedata.polissa')
        con_id = contract_obj.search([('name', '=', contract)])
        if not con_id:
            return jsonify({'status': 404, 'message': 'Not Found'}), 404
        contract = contract_obj.browse(con_id[0])
        if not profile_ids:
            mod = contract.modcontractual_activa.id
            data['modcontractual_id'] = mod
            profile_id = model.create(data)
        else:
            profile_id = profile_ids[0]
            model.write([profile_id], data)
        contract.write({'empowering_profile': profile_id})
        return jsonify({'status': 200, 'message': 'OK'}), 200


class EmpoweringSettingsDefForm(FormResource):
    def get(self):
        lang = get_first_lang()
        model = peek.model('giscedata.cups.ps')
        def_fields = model.fields_get(['empowering'], context={'lang': lang})
        res = jsonform(def_fields)
        return Response(json.dumps(res), mimetype='application/json')


class EmpoweringSettingsForm(FormResource, AuthorizedByContractResource):
    def get(self, contract):
        contract_obj = peek.model('giscedata.polissa')
        contract_ids = contract_obj.search([
            ('name', '=', contract)
        ])
        if not contract_ids:
            res = {}
        else:
            cups_id = contract_obj.read(contract_ids[0], ['cups'])['cups'][0]
            cups_obj = peek.model('giscedata.cups.ps')
            res = cups_obj.read(cups_id, ['empowering'])
        return Response(json.dumps(res), mimetype='application/json')

    def post(self, contract):
        data = request.json
        contract_obj = peek.model('giscedata.polissa')
        con_id = contract_obj.search([('name', '=', contract)])
        if not con_id:
            return jsonify({'status': 404, 'message': 'Not Found'}), 404
        cups_id = contract_obj.read(con_id, ['cups'])[0]['cups'][0]
   
        if 'empowering' not in data or not isinstance(data['empowering'], bool):  
            return jsonify({'status': 404, 'message': 'Data missing'}), 404

        cups_obj = peek.model('giscedata.cups.ps')
        cups_obj.write([cups_id], {'empowering': data['empowering']})
        return jsonify({'status': 200, 'message': 'OK'}), 200
resources = [
    (EmpoweringBuildingDefForm, '/EmpoweringBuildingDefForm'),
    (EmpoweringBuildingForm, '/EmpoweringBuildingForm/<cups>'),
    (EmpoweringProfileDefForm, '/EmpoweringProfileDefForm'),
    (EmpoweringProfileForm, '/EmpoweringProfileForm/<contract>'),
    (EmpoweringSettingsDefForm, '/EmpoweringSettingsDefForm'),
    (EmpoweringSettingsForm, '/EmpoweringSettingsForm/<contract>')
]



from flask import jsonify
from amoniak.caching import AVALIABLE_ONLINE_TOOLS

from heman.config import mongo
from heman.api.empowering.base import EmpoweringResource


class PeriodRange(EmpoweringResource):
    def get(self, contract):
        existing_collections = mongo.db.collection_names(False)
        result = {'max': None, 'min': None}
        for collection in AVALIABLE_ONLINE_TOOLS:
            if collection in existing_collections:
                collection = mongo.db[collection]
                cursor = collection.aggregate([
                    {"$match": {"contractId": contract}},
                    {"$group": {
                        "_id": "$item",
                        "minPeriod": {"$min": "$month"},
                        "maxPeriod": {"$max": "$month"}
                    }}
                ])
                if cursor.get('result'):
                    if result['max'] is None:
                        result['max'] = cursor['result'][0]['maxPeriod']
                    else:
                        result['max'] = max(
                            result['max'], cursor['result'][0]['maxPeriod']
                        )
                    if result['min'] is None:
                        result['min'] = cursor['result'][0]['minPeriod']
                    else:
                        result['min'] = min(
                            result['min'], cursor['result'][0]['minPeriod']
                        )
        return jsonify(result)



from flask import current_app, jsonify
from amoniak.caching import OT201Caching

from heman.config import mongo
from heman.api.empowering import service
from heman.api.empowering.base import EmpoweringResource


class OT201(EmpoweringResource):
    def get(self, contract, period):
        ot201 = OT201Caching(service, mongo.db)
        result = ot201.get_cached(contract, period)
        if not result.get('_items'):
            current_app.logger.debug(
                'No cached result for contract %s and period %s',
                contract, period
            )
        return jsonify(result)



# -*- coding: utf-8 -*-
from amoniak.utils import setup_empowering_api


class HeMan(object):
    def __init__(self, app=None):
        """Initializized a HeMan object.
        """
        self.app = None
        self.empowering = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        self.set_defaults()
        self.empowering = setup_empowering_api(**self.app.config)

    def set_defaults(self):
        self.app.config.setdefault('HEMAN_PREFIX', '/heman')



from heman.api import AuthorizedResource
from heman.auth import check_contract_allowed


class EmpoweringResource(AuthorizedResource):
    method_decorators = (
        AuthorizedResource.method_decorators + [check_contract_allowed]
    )



from __future__ import absolute_import
import logging
import os

from flask import Flask
from flask.ext.pymongo import PyMongo
from raven.contrib.flask import Sentry

from heman.api import HemanAPI
from amoniak.utils import setup_peek


api = HemanAPI(prefix='/api')
"""API object
"""
sentry = Sentry(logging=True, level=logging.ERROR)
"""Sentry object
"""
mongo = PyMongo()
"""Access to database

In other parts of the application you can do::

    from heman.config import mongo

    mongo.db.collection.find({"foo": "bar"})
"""

peek = setup_peek()
"""ERP Backend configuration
"""


def create_app(**config):
    """Application Factory

    You can create a new He-Man application with::

        from heman.config import create_app

        app = create_app() # app can be uses as WSGI application
        app.run() # Or you can run as a simple web server
    """
    app = Flask(
        __name__, static_folder=None
    )

    if 'MONGO_URI' in os.environ:
        app.config['MONGO_URI'] = os.environ['MONGO_URI']

    app.config['LOG_LEVEL'] = 'DEBUG'
    app.config['SECRET_KEY'] = '2205552d13b5431bb537732bbb051f1214414f5ab34d47'

    configure_logging(app)
    configure_sentry(app)
    configure_api(app)
    configure_mongodb(app)
    configure_login(app)

    return app


def configure_api(app):
    """Configure API Endpoints.
    """
    from heman.api.empowering import resources as empowering_resources
    from heman.api.cch import resources as cch_resources
    from heman.api.form import resources as form_resources
    from heman.api import ApiCatchall

    # Add Empowering resources
    for resource in empowering_resources:
        api.add_resource(*resource)

    # Add CCHFact resources
    for resource in cch_resources:
        api.add_resource(*resource)

    # Add Form resources
    for resource in form_resources:
        api.add_resource(*resource)

    api.add_resource(ApiCatchall, '/<path:path>')
    api.init_app(app)


def configure_sentry(app):
    """Configure Sentry logger.

    Uses `Raven
    <http://raven.readthedocs.org/en/latest/integrations/flask.html>`_
    """
    sentry.init_app(app)


def configure_mongodb(app):
    """Configure MongoDB access.

    Uses `Flask-PyMongo <https://flask-pymongo.readthedocs.org/>`_
    """
    mongo.init_app(app)


def configure_logging(app):
    """Configure logging

    Call ``logging.basicConfig()`` with the level ``LOG_LEVEL`` of application.
    """
    logging.basicConfig(level=getattr(logging, app.config['LOG_LEVEL']))


def configure_login(app):
    """Configure login authentification

    Uses `Flask-Login <https://flask-login.readthedocs.org>`_
    """
    from heman.auth import login_manager, login
    login_manager.init_app(app)

    @app.teardown_request
    def force_logout(*args, **kwargs):
        login.logout_user()



from datetime import datetime
from dateutil.relativedelta import relativedelta
import json
import time

from flask import current_app, jsonify, request, Response
from flask.ext.pymongo import ASCENDING

from heman.api import AuthorizedResource
from heman.auth import check_cups_allowed
from heman.config import mongo


class CCHResource(AuthorizedResource):
    method_decorators = (
        AuthorizedResource.method_decorators + [check_cups_allowed]
    )

    def options(self, *args, **kwargs):
        return jsonify({})


class CCHFact(CCHResource):
    def get(self, cups, period):
        interval = request.args.get('interval')
        try:
            interval = max(min(int(interval), 12), 1)
        except:
            interval = 12
        end = datetime.strptime(period, '%Y%m') + relativedelta(months=1)
        start = end - relativedelta(months=interval)
        res = []
        current_app.logger.debug('CCH from {} to {}'.format(start, end))
        search_query = {
            'name': cups,
            'datetime': {'$gte': start, '$lt': end}}
        if cups.endswith('0F'):
            search_query.pop('name', None)
            search_query['$or'] = [{'name': cups}, {'name': cups[:-2]}]
        cursor = mongo.db['tg_cchfact'].find(search_query,
            fields={'_id': False, 'datetime': True, 'ai': True}).sort(
            'datetime', ASCENDING
        )
        # Forcing local timezone

        for item in cursor:
            dt = item['datetime']
            dt_tuple = datetime(dt.year, dt.month, dt.day, dt.hour).timetuple()
            res.append({
                # Unix timestamp in Javascript is python * 1000
                'date': time.mktime(dt_tuple) * 1000,
                'value': item['ai']
            })
        return Response(json.dumps(res), mimetype='application/json')

resources = [
    (CCHFact, '/CCHFact/<cups>/<period>')
]



from __future__ import absolute_import
from functools import wraps

from flask import current_app
from flask.ext import login
from flask.ext.login import current_user

from heman.config import mongo


login_manager = login.LoginManager()
"""Login manager object
"""


def check_contract_allowed(func):
    """Check if Contract is allowed by token
    """
    @wraps(func)
    def decorator(*args, **kwargs):
        contract = kwargs.get('contract')
        if (contract and current_user.is_authenticated()
                and not current_user.allowed(contract)):
            return current_app.login_manager.unauthorized()
        return func(*args, **kwargs)
    return decorator


def check_cups_allowed(func):
    """Check if CUPS is allowd by token
    """
    @wraps(func)
    def decorator(*args, **kwargs):
        cups = kwargs.get('cups')
        if (cups and current_user.is_authenticated()
                and not current_user.allowed(cups, 'cups')):
            return current_app.login_manager.unauthorized()
        return func(*args, **kwargs)
    return decorator


class APIUser(login.UserMixin):
    """API User object

    :param token: token for this user
    """
    def __init__(self, token, allowed):
        self.id = token
        self.allowed_contracts = allowed

    def is_authenticated(self):
        return True

    def get_id(self):
        return self.id

    def allowed(self, value, key='name'):
        return value in [x[key] for x in self.allowed_contracts]


@login_manager.header_loader
def load_user_from_header(header_val):
    header_val = header_val.replace('Basic ', '', 1)
    try:
        user, token = header_val.split()
        if user == 'token':
            res = mongo.db.tokens.find({'token': token})
            if res:
                res = res[0]
                return APIUser(res['token'], res.get('allowed_contracts', []))
    except Exception:
        return None


@login_manager.user_loader
def load_user(token):
    return APIUser(token)



from __future__ import absolute_import

from flask import jsonify
from flask.ext import restful, login, cors


class HemanAPI(restful.Api):
    pass


class BaseResource(restful.Resource):
    """Base resource
    """
    def options(self, *args, **kwargs):
        return jsonify({})


class AuthorizedResource(BaseResource):
    """Autorized resource

    Base resource to inherit if the resource must be protected with auth
    """
    method_decorators = [login.login_required, cors.cross_origin()]


class ApiCatchall(BaseResource):
    def get(self, path):
        return jsonify({'status': 404, 'message': 'Not Found'}), 404

    post = get
    put = get
    delete = get
    patch = get



from setuptools import setup, find_packages

with open('requirements.txt', 'r') as req:
    INSTALL_REQUIRES = [x.strip() for x in req.readlines()]

with open('README.rst', 'r') as desc:
    DESCRIPTION = desc.read()

setup(
    name='heman',
    version='0.3.0',
    packages=find_packages(),
    url='https://github.com/gisce/heman',
    license='MIT',
    install_requires=INSTALL_REQUIRES,
    author='GISCE-TI, S.L.',
    author_email='ti@gisce.net',
    description=DESCRIPTION
)



from __future__ import absolute_import

from amoniak.utils import setup_empowering_api


service = setup_empowering_api()
"""Empowering service
"""

from heman.api.empowering.ot101 import OT101
from heman.api.empowering.ot103 import OT103
from heman.api.empowering.ot201 import OT201
from heman.api.empowering.ot401 import OT401
from heman.api.empowering.custom import PeriodRange

resources = [
    (OT101, '/OT101Results/<contract>/<period>'),
    (OT103, '/OT103Results/<contract>/<period>'),
    (OT201, '/OT201Results/<contract>/<period>'),
    (OT401, '/OT401Results/<contract>/<period>'),
    (PeriodRange, '/PeriodRange/<contract>')
]
